@@ -1,5 +1,5 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2015-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
@@ -40,7 +40,7 @@ void FeedScriptController::ExecuteScripts(const char* feedScript, const char* fe
 	}
 }
 
-void FeedScriptController::ExecuteScript(ScriptConfig::Script* script)
+void FeedScriptController::ExecuteScript(std::shared_ptr<const Extension::Script> script)
 {
 	if (!script->GetFeedScript())
 	{
@@ -49,7 +49,7 @@ void FeedScriptController::ExecuteScript(ScriptConfig::Script* script)
 
 	PrintMessage(Message::mkInfo, "Executing feed-script %s for Feed%i", script->GetName(), m_feedId);
 
-	SetArgs({script->GetLocation()});
+	SetArgs({script->GetEntry()});
 
 	BString<1024> infoName("feed-script %s for Feed%i", script->GetName(), m_feedId);
 	SetInfoName(infoName);
@@ -62,7 +62,7 @@ void FeedScriptController::ExecuteScript(ScriptConfig::Script* script)
 	if (exitCode != FEED_SUCCESS)
 	{
 		infoName[0] = 'F'; // uppercase
-		PrintMessage(Message::mkError, "%s failed", GetInfoName());
+		PrintMessage(Message::mkError, "%s failed", *infoName);
 		m_success = false;
 	}
 
