@@ -1,7 +1,8 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
- *  Copyright (C) 2014-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2014-2019 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2024-2025 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,11 +15,14 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 
 #include "nzbget.h"
+
+#include <sstream>
+#include <iomanip>
 #include "ArticleWriter.h"
 #include "DiskState.h"
 #include "Options.h"
@@ -54,7 +58,9 @@ void ArticleWriter::SetWriteBuffer(DiskFile& outFile, int recSize)
 void ArticleWriter::Prepare()
 {
 	BuildOutputFilename();
-	m_resultFilename = m_articleInfo->GetResultFilename();
+	m_resultFilename = m_articleInfo->GetResultFilename()
+		? m_articleInfo->GetResultFilename()
+		: "";
 }
 
 bool ArticleWriter::Start(Decoder::EFormat format, const char* filename, int64 fileSize,
@@ -106,31 +112,31 @@ bool ArticleWriter::Start(Decoder::EFormat format, const char* filename, int64 f
 	}
 
 	// allocate cache buffer
-	if (g_Options->GetArticleCache() > 0 && g_Options->GetDecode() &&
+	if (g_Options->GetArticleCache() > 0 && !g_Options->GetRawArticle() &&
 		(!g_Options->GetDirectWrite() || m_format == Decoder::efYenc))
 	{
 		m_articleData = g_ArticleCache->Alloc(m_articleSize);
 
 		while (!m_articleData.GetData() && g_ArticleCache->GetFlushing())
 		{
-			usleep(5 * 1000);
+			Util::Sleep(5);
 			m_articleData = g_ArticleCache->Alloc(m_articleSize);
 		}
 
 		if (!m_articleData.GetData())
 		{
-			detail("Article cache is full, using disk for %s", *m_infoName);
+			detail("Article cache is full, using disk for %s", m_infoName.c_str());
 		}
 	}
 
 	if (!m_articleData.GetData())
 	{
 		bool directWrite = (g_Options->GetDirectWrite() || m_fileInfo->GetForceDirectWrite()) && m_format == Decoder::efYenc;
-		const char* filename = directWrite ? m_outputFilename : m_tempFilename;
-		if (!m_outFile.Open(filename, directWrite ? DiskFile::omReadWrite : DiskFile::omWrite))
+		const char* outFilename = directWrite ? m_outputFilename.c_str() : m_tempFilename.c_str();
+		if (!m_outFile.Open(outFilename, directWrite ? DiskFile::omReadWrite : DiskFile::omWrite))
 		{
 			m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError,
-				"Could not %s file %s: %s", directWrite ? "open" : "create", filename,
+				"Could not %s file %s: %s", directWrite ? "open" : "create", outFilename,
 				*FileSystem::GetLastErrorMessage());
 			return false;
 		}
@@ -145,24 +151,43 @@ bool ArticleWriter::Start(Decoder::EFormat format, const char* filename, int64 f
 	return true;
 }
 
+bool ArticleWriter::GetSkipDiskWrite()
+{
+	if (m_fileInfo && m_fileInfo->GetNzbInfo() && m_fileInfo->GetNzbInfo()->GetSkipDiskWrite())
+	{
+		return true;
+	}
+
+	return g_Options->GetSkipWrite();
+}
+
 bool ArticleWriter::Write(char* buffer, int len)
 {
-	if (g_Options->GetDecode())
+	if (!g_Options->GetRawArticle())
 	{
 		m_articlePtr += len;
 	}
 
-	if (g_Options->GetDecode() && m_articleData.GetData())
+	if (m_articlePtr > m_articleSize)
+	{
+		// An attempt to write beyond article border is detected.
+		// That's an error condition (damaged article).
+		// We return 'false' since this isn't a fatal disk error and
+		// article size mismatch will be detected in decoder check anyway.
+		return true;
+	}
+
+	if (!g_Options->GetRawArticle() && m_articleData.GetData())
 	{
-		if (m_articlePtr > m_articleSize)
-		{
-			detail("Decoding %s failed: article size mismatch", *m_infoName);
-			return false;
-		}
 		memcpy(m_articleData.GetData() + m_articlePtr - len, buffer, len);
 		return true;
 	}
 
+	if (GetSkipDiskWrite())
+	{
+		return true;
+	}
+
 	return m_outFile.Write(buffer, len) > 0;
 }
 
@@ -172,27 +197,26 @@ void ArticleWriter::Finish(bool success)
 
 	if (!success)
 	{
-		FileSystem::DeleteFile(m_tempFilename);
-		FileSystem::DeleteFile(m_resultFilename);
+		FileSystem::DeleteFile(m_tempFilename.c_str());
+		FileSystem::DeleteFile(m_resultFilename.c_str());
 		return;
 	}
 
 	bool directWrite = (g_Options->GetDirectWrite() || m_fileInfo->GetForceDirectWrite()) && m_format == Decoder::efYenc;
 
-	if (g_Options->GetDecode())
+	if (!g_Options->GetRawArticle())
 	{
 		if (!directWrite && !m_articleData.GetData())
 		{
-			if (!FileSystem::MoveFile(m_tempFilename, m_resultFilename))
+			if (!FileSystem::MoveFile(m_tempFilename.c_str(), m_resultFilename.c_str()))
 			{
 				m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError,
-					"Could not rename file %s to %s: %s", *m_tempFilename, m_resultFilename,
+					"Could not rename file %s to %s: %s", m_tempFilename.c_str(), m_resultFilename.c_str(),
 					*FileSystem::GetLastErrorMessage());
 			}
+			FileSystem::DeleteFile(m_tempFilename.c_str());
 		}
 
-		FileSystem::DeleteFile(m_tempFilename);
-
 		if (m_articleData.GetData())
 		{
 			if (m_articleSize != m_articlePtr)
@@ -212,10 +236,10 @@ void ArticleWriter::Finish(bool success)
 	else
 	{
 		// rawmode
-		if (!FileSystem::MoveFile(m_tempFilename, m_resultFilename))
+		if (!FileSystem::MoveFile(m_tempFilename.c_str(), m_resultFilename.c_str()))
 		{
 			m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError,
-				"Could not move file %s to %s: %s", *m_tempFilename, m_resultFilename,
+				"Could not move file %s to %s: %s", m_tempFilename.c_str(), m_resultFilename.c_str(),
 				*FileSystem::GetLastErrorMessage());
 		}
 	}
@@ -224,19 +248,20 @@ void ArticleWriter::Finish(bool success)
 /* creates output file and subdirectores */
 bool ArticleWriter::CreateOutputFile(int64 size)
 {
-	if (g_Options->GetDirectWrite() && FileSystem::FileExists(m_outputFilename) &&
-		FileSystem::FileSize(m_outputFilename) == size)
+	if (FileSystem::FileExists(m_outputFilename.c_str()))
 	{
-		// keep existing old file from previous program session
-		return true;
+		if (FileSystem::FileSize(m_outputFilename.c_str()) == size)
+		{
+			// keep existing old file from previous program session
+			return true;
+		}
+		// delete existing old file from previous program session
+		FileSystem::DeleteFile(m_outputFilename.c_str());
 	}
 
-	// delete eventually existing old file from previous program session
-	FileSystem::DeleteFile(m_outputFilename);
-
 	// ensure the directory exist
 	BString<1024> destDir;
-	destDir.Set(m_outputFilename, FileSystem::BaseFileName(m_outputFilename) - m_outputFilename);
+	destDir.Set(m_outputFilename.c_str(), (int)(FileSystem::BaseFileName(m_outputFilename.c_str()) - m_outputFilename.c_str()));
 	CString errmsg;
 
 	if (!FileSystem::ForceDirectories(destDir, errmsg))
@@ -246,10 +271,10 @@ bool ArticleWriter::CreateOutputFile(int64 size)
 		return false;
 	}
 
-	if (!FileSystem::AllocateFile(m_outputFilename, size, g_Options->GetArticleCache() == 0, errmsg))
+	if (!FileSystem::AllocateFile(m_outputFilename.c_str(), size, g_Options->GetArticleCache() == 0, errmsg))
 	{
 		m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError,
-			"Could not create file %s: %s", *m_outputFilename, *errmsg);
+			"Could not create file %s: %s", m_outputFilename.c_str(), *errmsg);
 		return false;
 	}
 
@@ -258,30 +283,52 @@ bool ArticleWriter::CreateOutputFile(int64 size)
 
 void ArticleWriter::BuildOutputFilename()
 {
-	BString<1024> filename("%s%c%i.%03i", g_Options->GetTempDir(), PATH_SEPARATOR,
-		m_fileInfo->GetId(), m_articleInfo->GetPartNumber());
+	if (!m_fileInfo || !m_articleInfo)
+		return;
 
-	m_articleInfo->SetResultFilename(filename);
-	m_tempFilename.Format("%s.tmp", *filename);
+	const char* tmpDir = g_Options->GetTempDir();
+	const char* destDir = m_fileInfo->GetNzbInfo()->GetDestDir();
+	if (Util::EmptyStr(tmpDir) || Util::EmptyStr(destDir))
+		return;
 
+	std::stringstream ss;
+	ss << tmpDir;
+	ss << PATH_SEPARATOR;
+	ss << m_fileInfo->GetId();
+	ss << ".";
+	ss << std::setfill('0') << std::setw(3) << m_articleInfo->GetPartNumber();
+
+	std::string filename = ss.str();
+
+	m_articleInfo->SetResultFilename(filename.c_str());
+	m_tempFilename = filename + ".tmp";
+	
 	if (g_Options->GetDirectWrite() || m_fileInfo->GetForceDirectWrite())
 	{
 		Guard guard = m_fileInfo->GuardOutputFile();
+		const std::string& outputFilename = m_fileInfo->GetOutputFilename();
 
-		if (m_fileInfo->GetOutputFilename())
+		if (!outputFilename.empty())
 		{
-			filename = m_fileInfo->GetOutputFilename();
+			filename = outputFilename;
 		}
 		else
 		{
-			filename.Format("%s%c%i.out.tmp", m_fileInfo->GetNzbInfo()->GetDestDir(),
-				PATH_SEPARATOR, m_fileInfo->GetId());
+			ss.str("");
+			ss.clear();
+
+			ss << destDir;
+			ss << PATH_SEPARATOR;
+			ss << m_fileInfo->GetId();
+			ss << ".out.tmp";
+
+			filename = ss.str();
 			m_fileInfo->SetOutputFilename(filename);
 		}
 
-		m_outputFilename = *filename;
+		m_outputFilename = std::move(filename);
 	}
-}
+ }
 
 void ArticleWriter::CompleteFileParts()
 {
@@ -290,87 +337,108 @@ void ArticleWriter::CompleteFileParts()
 
 	bool directWrite = (g_Options->GetDirectWrite() || m_fileInfo->GetForceDirectWrite()) && m_fileInfo->GetOutputInitialized();
 
-	BString<1024> nzbName;
-	BString<1024> nzbDestDir;
-	BString<1024> filename;
+	std::string nzbDestDir;
+	std::string nzbName;
+	std::string destDir;
+	std::string filename;
 
 	{
 		GuardedDownloadQueue guard = DownloadQueue::Guard();
 		nzbName = m_fileInfo->GetNzbInfo()->GetName();
 		nzbDestDir = m_fileInfo->GetNzbInfo()->GetDestDir();
-		filename = m_fileInfo->GetFilename();
+		m_outputFilename = m_fileInfo->GetOutputFilename();
+		if (m_outputFilename.empty())
+		{
+			destDir = nzbDestDir;
+			filename = m_fileInfo->GetFilename();
+		}
+		else
+		{
+			auto [destDir_, filename_] = FileSystem::SplitPathAndFilename(m_outputFilename);
+			destDir.swap(destDir_);
+			filename.swap(filename_);
+		}
 	}
 
-	BString<1024> infoFilename("%s%c%s", *nzbName, PATH_SEPARATOR, *filename);
+	std::string infoFilename = nzbName + PATH_SEPARATOR + m_fileInfo->GetFilename();
 
 	bool cached = m_fileInfo->GetCachedArticles() > 0;
 
-	if (!g_Options->GetDecode())
+	if (g_Options->GetRawArticle())
 	{
-		detail("Moving articles for %s", *infoFilename);
+		detail("Moving articles for %s", infoFilename.c_str());
 	}
 	else if (directWrite && cached)
 	{
-		detail("Writing articles for %s", *infoFilename);
+		detail("Writing articles for %s", infoFilename.c_str());
 	}
 	else if (directWrite)
 	{
-		detail("Checking articles for %s", *infoFilename);
+		detail("Checking articles for %s", infoFilename.c_str());
 	}
 	else
 	{
-		detail("Joining articles for %s", *infoFilename);
+		detail("Joining articles for %s", infoFilename.c_str());
 	}
 
 	// Ensure the DstDir is created
 	CString errmsg;
-	if (!FileSystem::ForceDirectories(nzbDestDir, errmsg))
+	if (!FileSystem::ForceDirectories(destDir.c_str(), errmsg))
 	{
 		m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError,
-			"Could not create directory %s: %s", *nzbDestDir, *errmsg);
+			"Could not create directory %s: %s", destDir.c_str(), *errmsg);
 		return;
 	}
 
-	CString ofn;
+	std::string ofn;
 	if (m_fileInfo->GetForceDirectWrite())
 	{
-		ofn.Format("%s%c%s", *nzbDestDir, PATH_SEPARATOR, *filename);
+		ofn = destDir + PATH_SEPARATOR + infoFilename;
 	}
 	else
 	{
-		ofn = FileSystem::MakeUniqueFilename(nzbDestDir, *filename);
+		ofn = FileSystem::MakeUniqueFilename(destDir.c_str(), filename.c_str()).Str();
 	}
 
 	DiskFile outfile;
-	BString<1024> tmpdestfile("%s.tmp", *ofn);
+	std::string tmpdestfile = ofn + ".tmp";
 
-	if (g_Options->GetDecode() && !directWrite)
+	if (!g_Options->GetRawArticle() && !directWrite)
 	{
-		FileSystem::DeleteFile(tmpdestfile);
-		if (!outfile.Open(tmpdestfile, DiskFile::omWrite))
+		FileSystem::DeleteFile(tmpdestfile.c_str());
+		if (!outfile.Open(tmpdestfile.c_str(), DiskFile::omWrite))
 		{
 			m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError,
-				"Could not create file %s: %s", *tmpdestfile, *FileSystem::GetLastErrorMessage());
+				"Could not create file %s: %s",
+				tmpdestfile.c_str(),
+				*FileSystem::GetLastErrorMessage()
+			);
 			return;
 		}
 	}
 	else if (directWrite && cached)
 	{
-		if (!outfile.Open(m_outputFilename, DiskFile::omReadWrite))
+		if (!outfile.Open(m_outputFilename.c_str(), DiskFile::omReadWrite))
 		{
 			m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError,
-				"Could not open file %s: %s", *m_outputFilename, *FileSystem::GetLastErrorMessage());
+				"Could not open file %s: %s",
+				m_outputFilename.c_str(),
+				*FileSystem::GetLastErrorMessage()
+			);
 			return;
 		}
-		tmpdestfile = *m_outputFilename;
+		tmpdestfile = m_outputFilename;
 	}
-	else if (!g_Options->GetDecode())
+	else if (g_Options->GetRawArticle())
 	{
-		FileSystem::DeleteFile(tmpdestfile);
-		if (!FileSystem::CreateDirectory(ofn))
+		FileSystem::DeleteFile(tmpdestfile.c_str());
+		if (!FileSystem::CreateDirectory(ofn.c_str()))
 		{
 			m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError,
-				"Could not create directory %s: %s", *ofn, *FileSystem::GetLastErrorMessage());
+				"Could not create directory %s: %s",
+				ofn.c_str(),
+				*FileSystem::GetLastErrorMessage()
+			);
 			return;
 		}
 	}
@@ -392,7 +460,7 @@ void ArticleWriter::CompleteFileParts()
 		CharBuffer buffer;
 		bool firstArticle = true;
 
-		if (g_Options->GetDecode() && !directWrite)
+		if (!g_Options->GetRawArticle() && !directWrite)
 		{
 			buffer.Reserve(1024 * 64);
 		}
@@ -404,21 +472,27 @@ void ArticleWriter::CompleteFileParts()
 				continue;
 			}
 
-			if (g_Options->GetDecode() && !directWrite && pa->GetSegmentOffset() > -1 &&
+			if (!g_Options->GetRawArticle() && !directWrite && pa->GetSegmentOffset() > -1 &&
 				pa->GetSegmentOffset() > outfile.Position() && outfile.Position() > -1)
 			{
 				memset(buffer, 0, buffer.Size());
-				while (pa->GetSegmentOffset() > outfile.Position() && outfile.Position() > -1 &&
-					outfile.Write(buffer, std::min((int)(pa->GetSegmentOffset() - outfile.Position()), buffer.Size())));
+				if (!GetSkipDiskWrite())
+				{
+					while (pa->GetSegmentOffset() > outfile.Position() && outfile.Position() > -1 &&
+						outfile.Write(buffer, std::min((int)(pa->GetSegmentOffset() - outfile.Position()), buffer.Size())));
+				}
 			}
 
 			if (pa->GetSegmentContent())
 			{
-				outfile.Seek(pa->GetSegmentOffset());
-				outfile.Write(pa->GetSegmentContent(), pa->GetSegmentSize());
+				if (!GetSkipDiskWrite())
+				{
+					outfile.Seek(pa->GetSegmentOffset());
+					outfile.Write(pa->GetSegmentContent(), pa->GetSegmentSize());
+				}
 				pa->DiscardSegment();
 			}
-			else if (g_Options->GetDecode() && !directWrite)
+			else if (!g_Options->GetRawArticle() && !directWrite && !GetSkipDiskWrite())
 			{
 				DiskFile infile;
 				if (pa->GetResultFilename() && infile.Open(pa->GetResultFilename(), DiskFile::omRead))
@@ -437,13 +511,13 @@ void ArticleWriter::CompleteFileParts()
 					m_fileInfo->SetSuccessArticles(m_fileInfo->GetSuccessArticles() - 1);
 					m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError,
 						"Could not find file %s for %s [%i/%i]",
-						pa->GetResultFilename(), *infoFilename, pa->GetPartNumber(),
+						pa->GetResultFilename(), infoFilename.c_str(), pa->GetPartNumber(),
 						(int)m_fileInfo->GetArticles()->size());
 				}
 			}
-			else if (!g_Options->GetDecode())
+			else if (g_Options->GetRawArticle())
 			{
-				BString<1024> dstFileName("%s%c%03i", *ofn, PATH_SEPARATOR, pa->GetPartNumber());
+				BString<1024> dstFileName("%s%c%03i", ofn.c_str(), PATH_SEPARATOR, pa->GetPartNumber());
 				if (!FileSystem::MoveFile(pa->GetResultFilename(), dstFileName))
 				{
 					m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError,
@@ -454,7 +528,7 @@ void ArticleWriter::CompleteFileParts()
 
 			if (m_format == Decoder::efYenc)
 			{
-				crc = firstArticle ? pa->GetCrc() : Util::Crc32Combine(crc, pa->GetCrc(), pa->GetSegmentSize());
+				crc = firstArticle ? pa->GetCrc() : Crc32::Combine(crc, pa->GetCrc(), pa->GetSegmentSize());
 				firstArticle = false;
 			}
 		}
@@ -465,37 +539,14 @@ void ArticleWriter::CompleteFileParts()
 	if (outfile.Active())
 	{
 		outfile.Close();
-		if (!directWrite && !FileSystem::MoveFile(tmpdestfile, ofn))
+		if (!directWrite && !FileSystem::MoveFile(tmpdestfile.c_str(), ofn.c_str()))
 		{
 			m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError,
-				"Could not move file %s to %s: %s", *tmpdestfile, *ofn,
-				*FileSystem::GetLastErrorMessage());
-		}
-	}
-
-	if (directWrite)
-	{
-		if (!FileSystem::SameFilename(m_outputFilename, ofn) &&
-			!FileSystem::MoveFile(m_outputFilename, ofn))
-		{
-			m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError,
-				"Could not move file %s to %s: %s", *m_outputFilename, *ofn,
-				*FileSystem::GetLastErrorMessage());
-		}
-
-		// if destination directory was changed delete the old directory (if empty)
-		int len = strlen(nzbDestDir);
-		if (!(!strncmp(nzbDestDir, m_outputFilename, len) &&
-			(m_outputFilename[len] == PATH_SEPARATOR || m_outputFilename[len] == ALT_PATH_SEPARATOR)))
-		{
-			debug("Checking old dir for: %s", *m_outputFilename);
-			BString<1024> oldDestDir;
-			oldDestDir.Set(m_outputFilename, FileSystem::BaseFileName(m_outputFilename) - m_outputFilename);
-			if (FileSystem::DirEmpty(oldDestDir))
-			{
-				debug("Deleting old dir: %s", *oldDestDir);
-				FileSystem::RemoveDirectory(oldDestDir);
-			}
+				"Could not move file %s to %s: %s",
+				tmpdestfile.c_str(),
+				ofn.c_str(),
+				*FileSystem::GetLastErrorMessage()
+			);
 		}
 	}
 
@@ -512,62 +563,63 @@ void ArticleWriter::CompleteFileParts()
 
 	if (m_fileInfo->GetTotalArticles() == m_fileInfo->GetSuccessArticles())
 	{
-		m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkInfo, "Successfully downloaded %s", *infoFilename);
+		m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkInfo, "Successfully downloaded %s", infoFilename.c_str());
 	}
 	else if (m_fileInfo->GetMissedArticles() + m_fileInfo->GetFailedArticles() > 0)
 	{
 		m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkWarning,
 			"%i of %i article downloads failed for \"%s\"",
 			m_fileInfo->GetMissedArticles() + m_fileInfo->GetFailedArticles(),
-			m_fileInfo->GetTotalArticles(), *infoFilename);
-
-		if (g_Options->GetBrokenLog())
-		{
-			BString<1024> brokenLogName("%s%c_brokenlog.txt", *nzbDestDir, PATH_SEPARATOR);
-			DiskFile file;
-			if (file.Open(brokenLogName, DiskFile::omAppend))
-			{
-				file.Print("%s (%i/%i)%s", *filename, m_fileInfo->GetSuccessArticles(),
-					m_fileInfo->GetTotalArticles(), LINE_ENDING);
-				file.Close();
-			}
-		}
+			m_fileInfo->GetTotalArticles(),
+			infoFilename.c_str()
+		);
 	}
 	else
 	{
-		m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkInfo, "Partially downloaded %s", *infoFilename);
+		m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkInfo, "Partially downloaded %s", infoFilename.c_str());
 	}
 
 	{
 		GuardedDownloadQueue guard = DownloadQueue::Guard();
-
 		m_fileInfo->SetCrc(crc);
-		m_fileInfo->SetOutputFilename(ofn);
 
-		if (strcmp(m_fileInfo->GetFilename(), filename))
+		if (m_fileInfo->IsHardLinked() && Util::EmptyStr(g_Options->GetInterDir()))
 		{
-			// file was renamed during completion, need to move the file
-			ofn = FileSystem::MakeUniqueFilename(nzbDestDir, m_fileInfo->GetFilename());
-			if (!FileSystem::MoveFile(m_fileInfo->GetOutputFilename(), ofn))
+			// No need to rename, remove hardlink of .out.tmp-file
+			if (FileSystem::DeleteFile(m_outputFilename.c_str()))
 			{
-				m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError,
-					"Could not rename file %s to %s: %s", m_fileInfo->GetOutputFilename(),
-					*ofn, *FileSystem::GetLastErrorMessage());
+				m_fileInfo->SetOutputFilename(nullptr);
+				return;
+			}
+			else
+			{
+				m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError, "Cannot remove hardlink");
 			}
-			m_fileInfo->SetOutputFilename(ofn);
 		}
 
-		if (strcmp(m_fileInfo->GetNzbInfo()->GetDestDir(), nzbDestDir))
-		{
-			// destination directory was changed during completion, need to move the file
-			MoveCompletedFiles(m_fileInfo->GetNzbInfo(), nzbDestDir);
-		}
+		RenameOutputFile(filename, destDir);
 	}
 }
 
+void ArticleWriter::RenameOutputFile(const std::string& filename, const std::string& destDir)
+{
+	if (filename == m_fileInfo->GetFilename())
+		return;
+
+	std::string ofn = FileSystem::MakeUniqueFilename(destDir.c_str(), m_fileInfo->GetFilename()).Str();
+	if (!FileSystem::MoveFile(m_outputFilename.c_str(), ofn.c_str()))
+	{
+		m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError,
+			"Could not rename file %s to %s: %s",
+			m_outputFilename.c_str(), ofn.c_str(), *FileSystem::GetLastErrorMessage()
+		);
+	}
+	m_fileInfo->SetOutputFilename(std::move(ofn));
+}
+
 void ArticleWriter::FlushCache()
 {
-	detail("Flushing cache for %s", *m_infoName);
+	detail("Flushing cache for %s", m_infoName.c_str());
 
 	bool directWrite = g_Options->GetDirectWrite() && m_fileInfo->GetOutputInitialized();
 	DiskFile outfile;
@@ -579,10 +631,18 @@ void ArticleWriter::FlushCache()
 		ArticleCache::FlushGuard flushGuard = g_ArticleCache->GuardFlush();
 
 		std::vector<ArticleInfo*> cachedArticles;
-		cachedArticles.reserve(m_fileInfo->GetArticles()->size());
 
 		{
 			Guard contentGuard = g_ArticleCache->GuardContent();
+
+			if (m_fileInfo->GetFlushLocked())
+			{
+				return;
+			}
+
+			m_fileInfo->SetFlushLocked(true);
+
+			cachedArticles.reserve(m_fileInfo->GetArticles()->size());
 			for (ArticleInfo* pa : m_fileInfo->GetArticles())
 			{
 				if (pa->GetSegmentContent())
@@ -602,10 +662,11 @@ void ArticleWriter::FlushCache()
 
 			if (directWrite && !outfile.Active())
 			{
-				if (!outfile.Open(m_fileInfo->GetOutputFilename(), DiskFile::omReadWrite))
+				const std::string& outputFilename = m_fileInfo->GetOutputFilename();
+				if (!outfile.Open(outputFilename.c_str(), DiskFile::omReadWrite))
 				{
 					m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError,
-						"Could not open file %s: %s", m_fileInfo->GetOutputFilename(),
+						"Could not open file %s: %s", outputFilename.c_str(),
 						*FileSystem::GetLastErrorMessage());
 					// prevent multiple error messages
 					pa->DiscardSegment();
@@ -644,7 +705,10 @@ void ArticleWriter::FlushCache()
 				outfile.Seek(pa->GetSegmentOffset());
 			}
 
-			outfile.Write(pa->GetSegmentContent(), pa->GetSegmentSize());
+			if (!GetSkipDiskWrite())
+			{
+				outfile.Write(pa->GetSegmentContent(), pa->GetSegmentSize());
+			}
 
 			flushedSize += pa->GetSegmentSize();
 			flushedArticles++;
@@ -669,11 +733,12 @@ void ArticleWriter::FlushCache()
 		{
 			Guard contentGuard = g_ArticleCache->GuardContent();
 			m_fileInfo->SetCachedArticles(m_fileInfo->GetCachedArticles() - flushedArticles);
+			m_fileInfo->SetFlushLocked(false);
 		}
 	}
 
 	detail("Saved %i articles (%.2f MB) from cache into disk for %s", flushedArticles,
-		(float)(flushedSize / 1024.0 / 1024.0), *m_infoName);
+		(float)(flushedSize / 1024.0 / 1024.0), m_infoName.c_str());
 }
 
 bool ArticleWriter::MoveCompletedFiles(NzbInfo* nzbInfo, const char* oldDestDir)
@@ -712,57 +777,6 @@ bool ArticleWriter::MoveCompletedFiles(NzbInfo* nzbInfo, const char* oldDestDir)
 		}
 	}
 
-	// move brokenlog.txt
-	if (g_Options->GetBrokenLog())
-	{
-		BString<1024> oldBrokenLogName("%s%c_brokenlog.txt", oldDestDir, PATH_SEPARATOR);
-		if (FileSystem::FileExists(oldBrokenLogName))
-		{
-			BString<1024> brokenLogName("%s%c_brokenlog.txt", nzbInfo->GetDestDir(), PATH_SEPARATOR);
-
-			detail("Moving file %s to %s", *oldBrokenLogName, *brokenLogName);
-			if (FileSystem::FileExists(brokenLogName))
-			{
-				// copy content to existing new file, then delete old file
-				DiskFile outfile;
-				if (outfile.Open(brokenLogName, DiskFile::omAppend))
-				{
-					DiskFile infile;
-					if (infile.Open(oldBrokenLogName, DiskFile::omRead))
-					{
-						CharBuffer buffer(1024 * 50);
-						int cnt = buffer.Size();
-						while (cnt == buffer.Size())
-						{
-							cnt = (int)infile.Read(buffer, buffer.Size());
-							outfile.Write(buffer, cnt);
-						}
-						infile.Close();
-						FileSystem::DeleteFile(oldBrokenLogName);
-					}
-					else
-					{
-						nzbInfo->PrintMessage(Message::mkError, "Could not open file %s", *oldBrokenLogName);
-					}
-					outfile.Close();
-				}
-				else
-				{
-					nzbInfo->PrintMessage(Message::mkError, "Could not open file %s", *brokenLogName);
-				}
-			}
-			else
-			{
-				// move to new destination
-				if (!FileSystem::MoveFile(oldBrokenLogName, brokenLogName))
-				{
-					nzbInfo->PrintMessage(Message::mkError, "Could not move file %s to %s: %s",
-						*oldBrokenLogName, *brokenLogName, *FileSystem::GetLastErrorMessage());
-				}
-			}
-		}
-	}
-
 	// delete old directory (if empty)
 	if (FileSystem::DirEmpty(oldDestDir))
 	{
@@ -775,14 +789,15 @@ bool ArticleWriter::MoveCompletedFiles(NzbInfo* nzbInfo, const char* oldDestDir)
 				break;
 			}
 
+			const std::string& outputFilename = fileInfo->GetOutputFilename();
 			if (fileInfo->GetActiveDownloads() > 0)
 			{
 				Guard guard = fileInfo->GuardOutputFile();
-				pendingWrites = fileInfo->GetOutputInitialized() && !Util::EmptyStr(fileInfo->GetOutputFilename());
+				pendingWrites = fileInfo->GetOutputInitialized() && !outputFilename.empty();
 			}
 			else
 			{
-				pendingWrites = fileInfo->GetOutputInitialized() && !Util::EmptyStr(fileInfo->GetOutputFilename());
+				pendingWrites = fileInfo->GetOutputInitialized() && !outputFilename.empty();
 			}
 		}
 
@@ -798,7 +813,7 @@ bool ArticleWriter::MoveCompletedFiles(NzbInfo* nzbInfo, const char* oldDestDir)
 
 CachedSegmentData ArticleCache::Alloc(int size)
 {
-	Guard guard(m_allocMutex);
+	std::lock_guard<std::mutex> guard(m_allocMutex);
 
 	void* p = nullptr;
 
@@ -807,10 +822,15 @@ CachedSegmentData ArticleCache::Alloc(int size)
 		p = malloc(size);
 		if (p)
 		{
-			if (!m_allocated && g_Options->GetSaveQueue() && g_Options->GetServerMode() && g_Options->GetContinuePartial())
+			if (!m_allocated && g_Options->GetServerMode() && g_Options->GetContinuePartial())
 			{
 				g_DiskState->WriteCacheFlag();
 			}
+			if (!m_allocated)
+			{
+				// Resume Run(), the notification arrives later, after releasing m_allocMutex
+				m_allocCond.notify_all();
+			}
 			m_allocated += size;
 		}
 	}
@@ -820,13 +840,14 @@ CachedSegmentData ArticleCache::Alloc(int size)
 
 bool ArticleCache::Realloc(CachedSegmentData* segment, int newSize)
 {
-	Guard guard(m_allocMutex);
+	std::lock_guard<std::mutex> guard(m_allocMutex);
 
 	void* p = realloc(segment->m_data, newSize);
 	if (p)
 	{
 		m_allocated += newSize - segment->m_size;
 		segment->m_size = newSize;
+		segment->m_data = (char*)p;
 	}
 
 	return p;
@@ -838,9 +859,9 @@ void ArticleCache::Free(CachedSegmentData* segment)
 	{
 		free(segment->m_data);
 
-		Guard guard(m_allocMutex);
+		std::lock_guard<std::mutex> guard(m_allocMutex);
 		m_allocated -= segment->m_size;
-		if (!m_allocated && g_Options->GetSaveQueue() && g_Options->GetServerMode() && g_Options->GetContinuePartial())
+		if (!m_allocated && g_Options->GetServerMode() && g_Options->GetContinuePartial())
 		{
 			g_DiskState->DeleteCacheFlag();
 		}
@@ -856,21 +877,36 @@ void ArticleCache::Run()
 	bool justFlushed = false;
 	while (!IsStopped() || m_allocated > 0)
 	{
-		if ((justFlushed || resetCounter >= 1000  || IsStopped() ||
-			 (g_Options->GetDirectWrite() && m_allocated >= fillThreshold)) &&
+		if ((justFlushed || resetCounter >= 1000 || IsStopped() ||
+			(g_Options->GetDirectWrite() && m_allocated >= fillThreshold)) &&
 			m_allocated > 0)
 		{
 			justFlushed = CheckFlush(m_allocated >= fillThreshold);
 			resetCounter = 0;
 		}
+		else if (!m_allocated)
+		{
+			std::unique_lock<std::mutex> lk(m_allocMutex);
+			m_allocCond.wait(lk, [&] { return IsStopped() || m_allocated > 0; });
+			resetCounter = 0;
+		}
 		else
 		{
-			usleep(5 * 1000);
+			Util::Sleep(5);
 			resetCounter += 5;
 		}
 	}
 }
 
+void ArticleCache::Stop()
+{
+	Thread::Stop();
+
+	// Resume Run() to exit it
+	std::lock_guard<std::mutex> guard(m_allocMutex);
+	m_allocCond.notify_all();
+}
+
 bool ArticleCache::CheckFlush(bool flushEverything)
 {
 	debug("Checking cache, Allocated: %i, FlushEverything: %i", (int)m_allocated, (int)flushEverything);
