@@ -1,7 +1,8 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
- *  Copyright (C) 2014-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2014-2019 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2024-2025 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,13 +15,15 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 
 #ifndef ARTICLEWRITER_H
 #define ARTICLEWRITER_H
 
+#include <atomic>
+#include <string>
 #include "NString.h"
 #include "DownloadInfo.h"
 #include "Decoder.h"
@@ -48,7 +51,8 @@ private:
 class ArticleWriter
 {
 public:
-	void SetInfoName(const char* infoName) { m_infoName = infoName; }
+	void SetInfoName(const char* infoName) { m_infoName = infoName ? infoName : ""; }
+	void SetInfoName(std::string infoName) { m_infoName = std::move(infoName); }
 	void SetFileInfo(FileInfo* fileInfo) { m_fileInfo = fileInfo; }
 	void SetArticleInfo(ArticleInfo* articleInfo) { m_articleInfo = articleInfo; }
 	void Prepare();
@@ -60,23 +64,34 @@ public:
 	static bool MoveCompletedFiles(NzbInfo* nzbInfo, const char* oldDestDir);
 	void FlushCache();
 
-protected:
-	virtual void SetLastUpdateTimeNow() {}
-
 private:
+	bool GetSkipDiskWrite();
+
+	/**
+	 *  @brief Renames the temporary `.out` output file to its original filename.
+	 *  
+	 * 	This is necessary when Direct/Par renaming is disabled or the download fails due to health checks.
+	 *  Or there are no par files at all.
+	 *  No action is taken if the filename matches the current filename.
+	 *
+	 * @param filename The desired final filename (without path).
+	 * @param destDir  The destination directory for the file.
+	 */
+	void RenameOutputFile(const std::string& filename, const std::string& destDir);
+
 	FileInfo* m_fileInfo;
 	ArticleInfo* m_articleInfo;
 	DiskFile m_outFile;
-	CString m_tempFilename;
-	CString m_outputFilename;
-	const char* m_resultFilename = nullptr;
+	std::string m_tempFilename;
+	std::string m_outputFilename;
+	std::string m_resultFilename;
+	std::string m_infoName;
 	Decoder::EFormat m_format = Decoder::efUnknown;
 	CachedSegmentData m_articleData;
 	int64 m_articleOffset;
 	int m_articleSize;
 	int m_articlePtr;
 	bool m_duplicate = false;
-	CString m_infoName;
 
 	bool CreateOutputFile(int64 size);
 	void BuildOutputFilename();
@@ -98,6 +113,7 @@ public:
 	};
 
 	virtual void Run();
+	virtual void Stop();
 	CachedSegmentData Alloc(int size);
 	bool Realloc(CachedSegmentData* segment, int newSize);
 	void Free(CachedSegmentData* segment);
@@ -108,12 +124,13 @@ public:
 	bool FileBusy(FileInfo* fileInfo) { return fileInfo == m_fileInfo; }
 
 private:
-	size_t m_allocated = 0;
-	bool m_flushing = false;
-	Mutex m_allocMutex;
+	std::atomic<size_t> m_allocated{0};
+	FileInfo* m_fileInfo = nullptr;
+	std::mutex m_allocMutex;
+	std::condition_variable m_allocCond;
 	Mutex m_flushMutex;
 	Mutex m_contentMutex;
-	FileInfo* m_fileInfo = nullptr;
+	bool m_flushing = false;
 
 	bool CheckFlush(bool flushEverything);
 };
