@@ -1,8 +1,9 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2019 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2024 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 
@@ -31,7 +32,7 @@ int FileInfo::m_idMax = 0;
 int NzbInfo::m_idGen = 0;
 int NzbInfo::m_idMax = 0;
 DownloadQueue* DownloadQueue::g_DownloadQueue = nullptr;
-bool DownloadQueue::g_Loaded = false;
+std::atomic<bool> DownloadQueue::g_Loaded{false};
 
 void NzbParameterList::SetParameter(const char* name, const char* value)
 {
@@ -40,7 +41,7 @@ void NzbParameterList::SetParameter(const char* name, const char* value)
 	iterator pos = std::find_if(begin(), end(),
 		[name](NzbParameter& parameter)
 		{
-			return !strcmp(parameter.GetName(), name);
+			return !strcasecmp(parameter.GetName(), name);
 		});
 
 	if (emptyVal && pos != end())
@@ -57,12 +58,11 @@ void NzbParameterList::SetParameter(const char* name, const char* value)
 	}
 }
 
-NzbParameter* NzbParameterList::Find(const char* name, bool caseSensitive)
+NzbParameter* NzbParameterList::Find(const char* name)
 {
 	for (NzbParameter& parameter : this)
 	{
-		if ((caseSensitive && !strcmp(parameter.GetName(), name)) ||
-			(!caseSensitive && !strcasecmp(parameter.GetName(), name)))
+		if (!strcasecmp(parameter.GetName(), name))
 		{
 			return &parameter;
 		}
@@ -347,10 +347,12 @@ void NzbInfo::UpdateMinMaxTime()
 	}
 }
 
-void NzbInfo::AddMessage(Message::EKind kind, const char * text)
+void NzbInfo::AddMessage(Message::EKind kind, const char * text, bool print)
 {
-	switch (kind)
+	if (print)
 	{
+		switch (kind)
+		{
 		case Message::mkDetail:
 			detail("%s", text);
 			break;
@@ -370,19 +372,20 @@ void NzbInfo::AddMessage(Message::EKind kind, const char * text)
 		case Message::mkDebug:
 			debug("%s", text);
 			break;
+		}
 	}
 
 	Guard guard(m_logMutex);
 
 	m_messages.emplace_back(++m_idMessageGen, kind, Util::CurrentTime(), text);
 
-	if (g_Options->GetSaveQueue() && g_Options->GetServerMode() && g_Options->GetNzbLog())
+	if (g_Options->GetServerMode() && g_Options->GetNzbLog())
 	{
 		g_DiskState->AppendNzbMessage(m_id, kind, text);
-		m_messageCount++;
+		m_messageCount.fetch_add(1, std::memory_order_relaxed);
 	}
 
-	while (m_messages.size() > (uint32)g_Options->GetLogBufferSize())
+	while (m_messages.size() > (uint32)g_Options->GetLogBuffer())
 	{
 		m_messages.pop_front();
 	}
@@ -497,6 +500,9 @@ bool NzbInfo::IsDupeSuccess()
 		m_parStatus == NzbInfo::psFailure ||
 		m_unpackStatus == NzbInfo::usFailure ||
 		m_unpackStatus == NzbInfo::usPassword ||
+		m_urlStatus == NzbInfo::lsFailed ||
+		m_urlStatus == NzbInfo::lsScanSkipped ||
+		m_urlStatus == NzbInfo::lsScanFailed ||
 		(m_parStatus == NzbInfo::psSkipped &&
 		 m_unpackStatus == NzbInfo::usSkipped &&
 		 CalcHealth() < CalcCriticalHealth(true)));
@@ -629,6 +635,10 @@ const char* NzbInfo::MakeTextStatus(bool ignoreScriptStatus)
 		{
 			status = "DELETED/DUPE";
 		}
+		else if (m_deleteStatus == NzbInfo::dsGood)
+		{
+			status = "DELETED/GOOD";
+		}
 		else
 		{
 			const char* urlStatusName[] = { "FAILURE/INTERNAL_ERROR", "FAILURE/INTERNAL_ERROR", "FAILURE/INTERNAL_ERROR",
@@ -811,7 +821,7 @@ void FileInfo::SetExtraPriority(bool extraPriority)
 
 void FileInfo::MakeValidFilename()
 {
-	m_filename = FileSystem::MakeValidFilename(m_filename);
+	m_filename = FileSystem::MakeValidFilename(m_filename.c_str()).Str();
 }
 
 void FileInfo::SetActiveDownloads(int activeDownloads)
@@ -828,11 +838,22 @@ void FileInfo::SetActiveDownloads(int activeDownloads)
 	}
 }
 
+bool FileInfo::IsHardLinked()
+{
+	return !m_hardLinkPath.empty();
+}
 
-CompletedFile::CompletedFile(int id, const char* filename, EStatus status, uint32 crc,
-	bool parFile, const char* hash16k, const char* parSetId) :
-	m_id(id), m_filename(filename), m_status(status), m_crc(crc), m_parFile(parFile),
-	m_hash16k(hash16k), m_parSetId(parSetId)
+
+CompletedFile::CompletedFile(int id, std::string filename, std::string origname, EStatus status,
+	uint32 crc, bool parFile, std::string hash16k, std::string parSetId) 
+	: m_id(id)
+	, m_filename(std::move(filename))
+	, m_origname(std::move(origname))
+	, m_status(status)
+	, m_crc(crc)
+	, m_parFile(parFile)
+	, m_hash16k(std::move(hash16k))
+	, m_parSetId(std::move(parSetId))
 {
 	if (FileInfo::m_idMax < m_id)
 	{
