@@ -205,6 +205,9 @@ public:
 	void SetParSetId(const char* parSetId) { m_parSetId = parSetId; }
 	bool GetFlushLocked() { return m_flushLocked; }
 	void SetFlushLocked(bool flushLocked) { m_flushLocked = flushLocked; }
+	const std::string& GetHardLinkPath() const { return m_hardLinkPath; }
+	void SetHardLinkPath(std::string hardLinkPath) { m_hardLinkPath = std::move(hardLinkPath); }
+	bool IsHardLinked();
 
 	ServerStatList* GetServerStats() { return &m_serverStats; }
 
@@ -246,6 +249,7 @@ private:
 	CString m_hash16k;
 	CString m_parSetId;
 	bool m_flushLocked = false;
+	std::string m_hardLinkPath;
 
 	static int m_idGen;
 	static int m_idMax;
@@ -647,7 +651,7 @@ public:
 	const char* MakeTextStatus(bool ignoreScriptStatus);
 	void AddMessage(Message::EKind kind, const char* text, bool print = true);
 	void PrintMessage(Message::EKind kind, const char* format, ...) PRINTF_SYNTAX(3);
-	int GetMessageCount() { return m_messageCount; }
+	int GetMessageCount() const { return m_messageCount; }
 	void SetMessageCount(int messageCount) { m_messageCount = messageCount; }
 	int GetCachedMessageCount() { return m_cachedMessageCount; }
 	GuardedMessageList GuardCachedMessages() { return GuardedMessageList(&m_messages, &m_logMutex); }
@@ -761,7 +765,7 @@ private:
 	bool m_changed = false;
 	time_t m_queueScriptTime = 0;
 	bool m_parFull = false;
-	int m_messageCount = 0;
+	std::atomic<int> m_messageCount{0};
 	int m_cachedMessageCount = 0;
 	int m_feedId = 0;
 	bool m_allFirst = false;
