@@ -1,8 +1,8 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2005 Bo Cordes Petersen <placebodk@sourceforge.net>
- *  Copyright (C) 2007-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2019 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -497,7 +497,7 @@ bool RemoteClient::RequestServerList(bool files, bool groups, const char* patter
 			}
 
 			printf("-----------------------------------\n");
-			printf("Groups: %i\n", downloadQueue->GetQueue()->size());
+			printf("Groups: %i\n", (int)downloadQueue->GetQueue()->size());
 			if (pattern)
 			{
 				printf("Matches: %i\n", matches);
@@ -797,7 +797,7 @@ bool RemoteClient::RequestServerEditQueue(DownloadQueue::EEditAction action, int
 
 	if (textLen > 0)
 	{
-		strcpy(trailingData, text);
+		strncpy(trailingData, text, textLen);
 	}
 
 	int32* ids = (int32*)(trailingData + textLen);
@@ -812,7 +812,8 @@ bool RemoteClient::RequestServerEditQueue(DownloadQueue::EEditAction action, int
 		char *names = trailingData + textLen + idLength;
 		for (CString& name : nameList)
 		{
-			int len = strlen(name);
+			// "len" must be less or equal than: "buffer size" - "already used buffer" - "ending \0"
+			size_t len = strnlen(name, length - (names - trailingData) - 1);
 			strncpy(names, name, len + 1);
 			names += len + 1;
 		}
