@@ -1,13 +1,13 @@
 nzbget_options = ['ParCheck=auto', 'ParQuick=no', 'PostStrategy=rocket']
 
 def test_parchecker_healthy(nserv, nzbget):
-	hist = nzbget.download_nzb('parchecker.nzb')
+	hist = nzbget.download_nzb('parchecker2.nzb')
 	assert hist['Status'] == 'SUCCESS/HEALTH'
 
 def test_parchecker_repair(nserv, nzbget):
 	nzbget.api.pausepost();
-	nzb_content = nzbget.load_nzb('parchecker.nzb')
-	nzb_content = nzb_content.replace('parchecker/testfile.dat?1=0:3000', 'parchecker/testfile.dat?1=0:3000!0')
+	nzb_content = nzbget.load_nzb('parchecker2.nzb')
+	nzb_content = nzb_content.replace('parchecker2/testfile.7z.001?16=45000:3000', 'parchecker2/testfile.7z.001?16=45000:3000!0')
 	nzbget.append_nzb('parchecker.1.nzb', nzb_content, dupemode='FORCE')
 	nzbget.append_nzb('parchecker.2.nzb', nzb_content, dupemode='FORCE')
 	nzbget.append_nzb('parchecker.3.nzb', nzb_content, dupemode='FORCE')
